<?php
	require '../tmp/sql.php';
	require '../classes/mysql.php';
	
	$sql = new MySQL(); if ($sql->debug) error_reporting (E_ALL|E_STRICT); else error_reporting (0);
	
	require '../plugins/phpQuery.php';
	require '../classes/settings.php';
	require '../classes/security.php';
	require '../classes/functions.php';
	require '../classes/template.php';
	require '../classes/player.php';
	require '../classes/stats.php';
	
	$secure = new Security();
	$settings = new Settings();
	$player = new Player();
	$tpl = new Templates();
	$tpl->load_phrases("admin", true);
	date_default_timezone_set($settings->get("timezone"));
	
	
	### PLATOON-SYNCRONISATION ###
	// Check ob Platoon-Sync aktiviert ist
	if ($settings->get("platoon_sync_option")) {
		// Platoon-Sync ist aktiviert
		
		/* Check ob eine Syncronisation ansteht:
		   Wenn die Anzahl der Aufrufe dieser Updatefunktion grer/gleich der Syncronisationsfrequenz
		   ist, wird Syncronisiert. Ansonsten wird der Counter um 1 hochgesetzt */
		if ($settings->get("platoon_sync_count") >= $settings->get("platoon_sync_frequency")) {
			// Es steht eine Syncronisation an
			
			/* Check ob Platoons eingetragen wurden
			   Wenn nicht, geht es ohne weitere Aktion mit dem Update der Spieler weiter */
			$sql->query('SELECT * FROM '.$sql->prefix.'platoons ORDER BY platoonname');
			
			if ($sql->count()) {
				// Es wurden Platoons angegeben
				$settings->set("platoon_sync_count", "1", true); // Der Counter auf 1 zurckgesetzt
				$player_in_platoons = Array();  // Array fr alle Spielernamen aus allen Platoons
				$platform_of_players = Array(); // Array fr Plattformen der Spieler aus allen Platoons
				$player_in_database = Array();  // Array fr alle Spielernamen aus der Datenbank
				$message = Array(); // Array fr alle Aktionsmeldungen
				
				// Schleife durch alle angegebenen Platoons
				while ($row = $sql->fetchRow()) {
					// cURL-Abruf 
					$c=curl_init('http://battlelog.battlefield.com/bf3/platoon/'.$row['platoonid'].'/listmembers/');
					curl_setopt($c,CURLOPT_HEADER,false); curl_setopt($c,CURLOPT_HTTPHEADER,array('X-AjaxNavigation: 1')); curl_setopt($c,CURLOPT_RETURNTRANSFER,true);
					$result = json_decode(curl_exec($c));
					curl_close($c);
					
					// Array der Spieler des akuellen Platoons
					$players = $result->context->listMembers;
					
					// Check ob Battlelog berhaupt erreichbar ist
					if ($players === NULL) die("It seems that Battlelog is not available at the moment");
					
					// Alle Spielernamen und die Plattformen in die Arrays speichern
					foreach ($players as $playerobj) {
						if ($playerobj->membershipLevel >= 4) { // Membershiplevel: 2 (gelscht), 4 (Members), 128 (Leader), 256 (Founder)
							$player_in_platoons[] = $playerobj->persona->personaName;
							$platform_of_players[] = $row['platform'];
						}
					}
				}
				
				// Spieler aus der Datenbank in das Array speichern
				$sql->query('SELECT id, name FROM '.$sql->prefix.'player WHERE id != "1" ORDER BY name');
				while ($row = $sql->fetchRow()) $player_in_database[$row['id']] = $row['name'];
				
				// Spieler die im Platoon sind, aber nicht in der Datenbank, zu dieser hinzufgen
				for($i = 0; $i < count($player_in_platoons); $i++) {
					$name = $player_in_platoons[$i];
					$platform = $platform_of_players[$i];
					
					if (!in_array($name, $player_in_database)) {
						$message[] = $player->add($name, $platform);
					}
				}
				
				// Spieler, die in der Datenbank sind, aber in keinem Platoon, aus der DB lschen
				if ($settings->get("platoon_sync_option") == "2") {
					foreach ($player_in_database as $id => $name) {
						if (!in_array($name, $player_in_platoons)) {
							$message[] = $player->delete($id, $name);
						}
					}
				}
				
				// Ergebnis ausgeben
				if (isset ($_POST['request']) AND $_POST['request'] == "admin") {
					// Abruf aus dem Adminpanel heraus
					die (json_encode(array(
						"message" => $tpl->phrase('platoon_sync_complete')."<br/>".implode(", ", $message),
						"time" => date("d.m.Y - H:i", time())
					)));
				} 
				else {
					// Direkter Abruf
					eval ("\$html = \"".$tpl->template("update")."\";");
					$html = phpQuery::newDocumentHTML($html);
					
					pq('ul')->append('<li>'.$tpl->phrase('platoon_sync_complete').'</li>');
					
					foreach ($message as $string) pq('ul')->append('<li>'.$string.'</li>');
					die ($html);
				}
			}
		} else {
			// Es steht keine Syncronisation an
			$count = $settings->get("platoon_sync_count");
			$settings->set("platoon_sync_count", $count+1, true); // Der Counter wird um 1 erhht
		}
	}
	
	
	### UPDATE DER SPIELER ###
	### UPDATE-LOGIK ###
	/*
		- Es werden $settings->updatecount Spieler auf einmal aktualisiert
		- Es werden erst die Spieler aktualisiert, bei denen $player->pushed = 1 ist
		- Danach werden die aktualisiert, deren $player->last_update am kleinsten ist
		
		- Spieler die gepusht sind, erhalten auf jeden Fall $player->update()                                                      / $player->pushed = 0
		- Spieler, deren $player->checkstate = "0" oder "notfound" ist, erhalten $player->push_sync()                              / $player->pushed = 1
		- Spieler, deren $player->date_check lnger zurck liegt, als $settings->synctime Stunden, erhalten $player->push_sync()   / $player->pushed = 1
		- Die restlichen Spieler erhalten $player->update()                                                                        / $player->pushed = 0
	*//*
		$result = Array(
			checkstate,
			update_left,
			push
		);
	*/
	
	
	// Standard-Variablen belegen
	$message = Array();
	$players = Array();
	$update_left = "unknown";
	$time = time();
	$synctime = (int)$settings->get('synctime'); $synctime = $synctime * 3600;
	if (isset($_POST['request']) AND $_POST['request'] == "admin") $type = "direct"; else $type = "cronjob"; // Wenn das Update aus dem Adminpanel heraus gestartet wird, wird es mit der API-Option "type=direct" belegt, und wird mit hherer Prioritt abgearbeitet
	
	
	// Abrufen und Speichern von IDENT und KEY fr den bf3stats-Server, wenn die Daten nicht whrend der Installation eingegeben wurden 
	// Das kann passieren, wenn der bf3stats.com-Server down ist
	if ($settings->get("key") == "" OR $settings->get("key") == "invalid_name" OR $settings->get("key") == "invalid_clientident") {
		$ch = curl_init("http://curl.ezstats.org/");
		curl_setopt($ch, CURLOPT_POST, true); curl_setopt($ch, CURLOPT_RETURNTRANSFER, true); curl_setopt($ch, CURLOPT_POSTFIELDS, "test=curl&game=bf3&version=".$settings->get('version')."*&baseurl=".$_SERVER['HTTP_HOST']."&scripturl=".$_SERVER['HTTP_HOST']."/".str_replace($_SERVER['DOCUMENT_ROOT'], "", $_SERVER['SCRIPT_FILENAME']));
		$data = curl_exec($ch); curl_close($ch);
		$data = explode('||', $data);
		
		$settings->set("ident", $data[0], true);
		$settings->set("key", $data[1], true);
	}
	
	// Spieler abrufen, die gepusht sind
	$sql->query('
		SELECT id, name, platform, date_check, pushed, checkstate
		FROM '.$sql->prefix.'player
		WHERE id != 1
		AND pushed = "1"
		LIMIT '.$settings->get('updatecount').'
	');
	
	while ($row = $sql->fetchRow()) {
		$players[] = $row;
	}
	
	$left = $settings->get('updatecount') - count($players);
	if ($left > 0) {
		// Spieler abrufen, die am lngsten nicht mehr aktualisiert wurden
		$sql->query('
			SELECT id, name, platform, date_check, pushed, checkstate
			FROM '.$sql->prefix.'player
			WHERE id != 1
			ORDER BY last_update ASC
			LIMIT '.$left.'
		');
		
		while ($row = $sql->fetchRow()) {
			$players[] = $row;
		}
	}
	
	
	// Spieler aktualisieren
	foreach ($players as $array) {
		### Spieler war gepusht ###
		if ($array['pushed'] == "1") {
			$result = $player->update($array['id'], $array['name'], $array['platform']);
		}
		
		### Spieler ist noch nicht in der bf3stats-DB vorhanden ###
		else if ($array['checkstate'] == "0" OR $array['checkstate'] == "notfound") {
			$result = $player->push_sync($array['name'], $array['platform'], "direct");
		}
		
		### Letztes Update liegt lnger als "synctime" Stunden zurck
		else if (($time - $array['date_check']) > $synctime) {
			$result = $player->push_sync($array['name'], $array['platform'], $type);
		}
		
		### Spieler wird mit den bf3stats-Daten abgeglichen
		else {
			$result = $player->update($array['id'], $array['name'], $array['platform']);
		}
		
		
		// "update_left" protokollieren
		if($result['update_left'] !== NULL) $update_left = $result['update_left'];
		
		// Daten der Aktualisierung in die Spielereigenschaften schreiben
		$sql->query('
			UPDATE `'.$sql->prefix.'player` SET
				`last_update` = "'.$time.'",
				`checkstate`  = "'.$result['checkstate'].'",
				`pushed`      = "'.$result['push'].'"
			WHERE `id` = "'.$array['id'].'"
		');
		
		// Ergebnisnachricht schreiben
		$message[] = $array['name'].' ('.$result['checkstate'].')';
	}
	
	
	// "update_left" protokollieren
	$message[] = $tpl->phrase('update_left').$update_left;
	
	// Datum der Aktualisierung in die Settings schreiben
	$sql->query('UPDATE '.$sql->prefix.'settings SET value="'.$time.'" WHERE name = "lastcheckall"');
	
	
	// Ergebnis ausgeben
	if (isset ($_POST['request']) AND $_POST['request'] == "admin") {
		// Abruf aus dem Adminpanel heraus
		die (json_encode(array(
			"message" => $tpl->phrase('update_message').implode(", ", $message),
			"time" => date("d.m.Y - H:i", $time)
		)));
	} 
	else {
		// Direkter Abruf
		eval ("\$html = \"".$tpl->template("update")."\";");
		$html = phpQuery::newDocumentHTML($html);
		
		foreach ($message as $string) pq('ul')->append('<li>'.$string.'</li>');
		echo $html;
	}
?>